#ifndef ZBCRH
#define ZBCRH

#define LIB_MAJOR         1
#define LIB_MINOR         3
#define LIB_MICRO         29
#define LIB_PATCH         2

#ifdef WIN32
#define ZBCR_CALL  __stdcall
#define ZBCR_EXP   
#else
#define ZBCR_CALL   
#define ZBCR_EXP __attribute__ ((visibility("default")))
#endif

#ifdef __cplusplus
extern "C" {
#endif
typedef int  ZBCR_STATUS ;


/*
    解码成功回调函数
*/
typedef void (ZBCR_CALL *CALL_BACK_DATA)(unsigned char *data,unsigned int length, unsigned long userdata);

#define EVENT_DOWNLOAD_PRECENT      0x1001
/*
    接口事件回调
*/
typedef void (ZBCR_CALL *CALL_BACK_EVENT)(unsigned int event, unsigned long data, unsigned long userdata);

/*
    端口参数
*/
typedef struct portParam
{
    short portNo;
    unsigned long baudRate;
}PortParam;


/*
    连接扫描设备

    @param
        pParam  指向端口PortParam数据的指针. 
                NULL 自动连接扫描设备
                不为空 portNo
                    0 - 1000 串口号 
                    1100     HID端口
                    -1       自动连接扫描设备
                    -2       自动连接串口,轮询所有串口
    @return  
            ZBCR_SUCCESS    打开成功
            ZBCR_INIT_FAIL  打开失败
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_Open(void* pParam, CALL_BACK_DATA pfnReadData, unsigned long userdata);

/*
    连接扫描设备

    @param
        portNo  
                0 - 1000 串口号 
                1100     HID端口
                -1       自动连接扫描设备
                -2       自动连接串口,轮询所有串口
        baudRate
                串口波特率
        pfnReadData
                解码回调函数指针
        userdata
                用户数据句柄等，回调函数传入
    @return  
            ZBCR_SUCCESS    打开成功
            ZBCR_INIT_FAIL  打开失败
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_Open2(short portNo, long baudRate, CALL_BACK_DATA pfnReadData, unsigned long userdata);

/*
    连接扫描设备 用于linux系统下输入设备节点地址

    @param
        portAddr      
                端口地址
        baudRate
                串口波特率
        pfnReadData
                解码回调函数指针
        userdata
                用户数据句柄等，回调函数传入
    @return  
            ZBCR_SUCCESS    打开成功
            ZBCR_INIT_FAIL  打开失败
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_Open4(const char* portAddr, long baudRate,CALL_BACK_DATA pfnReadData, unsigned long userdata);

/*
    关闭设备连接，释放相关资源
    @param
    @return     1       
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_Close(void);


/*
    设置扫描设备扫码工作模式
    @param
        mode  工作模式类型
    @return  
            1    成功
            <0   失败
*/
typedef enum {
    LEVEL=0,            //按键按下开始解码，松开按键或解码成功停止解码
    TOGGLE=1,           //按键按下开始解码，再次按键按下停止解码
    AUTO=2,             //上电后一直解码
    PULSE=4,            //波动解码，按键按下开始解码，直到达到超时时间或解码成功停止解码
    MOTION_SENSOR=5     //扫描装置固定，图像中物体移动感应解码
} SCAN_MODE;
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_ScanMode(unsigned int mode);
/*
    主动开启扫码，主要用于 工作模式 是SCAN_MODE_LEVEL
    @param
        
    @return  
            1    成功
            0   失败
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_StartScan(void);
/*
    停止扫码
    @param
        
    @return  
            1    成功
            0   失败
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_StopScan(void);
/*
    读取解码数据
    @param
        nTimeOut            等待读取超时时长(单位 100ms)
        pOutputBuffer       输出获取的解码数据缓存
        nSizeOfOutputBuffer 缓存空间大小
        pnBytesReturned     输出实际解码数据长度
    @return  
            1    成功
            0    无数据
*/
// totalTimeOunt = nTimeOut*100ms
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_ReadDecode(unsigned short nTimeOut,
							unsigned char* pOutputBuffer, 
                            unsigned int nSizeOfOutputBuffer,
                            unsigned int *pnBytesReturned);



ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_SetupRead(unsigned char* pInputBuffer,
									    unsigned int nBytesInInputBuffer,
									    unsigned char *pOutputBuffer, 
								        unsigned int nSizeOfOutputBuffer, 
								        unsigned int *pnBytesReturned);


ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_SetupWrite(unsigned char* pInputBuffer,
									     unsigned int nBytesInInputBuffer,
									     unsigned char *pOutputBuffer, 
									     unsigned int nSizeOfOutputBuffer, 
                                         unsigned int *pnBytesReturned);


ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_ControlCmd (unsigned char *pInputBuffer,
									  unsigned int nBytesInInputBuffer,
									  unsigned char *pOutputBuffer,
									  unsigned int nSizeOfOutputBuffer,
                                      unsigned int *pnBytesReturned);


ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_StatusRead(  unsigned char* pInputBuffer,
									   unsigned int nBytesInInputBuffer,
									   unsigned char *pOutputBuffer, 
									   unsigned int nSizeOfOutputBuffer, 
									   unsigned int *pnBytesReturned);

ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_Command(unsigned char* pInputBuffer,
                                               unsigned int nBytesInInputBuffer,
                                               unsigned char *pOutputBuffer, 
                                               unsigned int nSizeOfOutputBuffer, 
                                               unsigned int *pnBytesReturned);
/*
  获取最后一张图像数据大小，可用于分配存储图像数据空间

  @param
  @return  
            > 0 图像数据大小
            -1  获取失败
*/
ZBCR_EXP int  ZBCR_CALL ZBCR_GetLastImageSize(void);

/*
  获取最后一张图像数据

  @param
            image_data  图像数据缓存指针，指向缓存空间，不可为空
            imagesize   [输入] 缓存空间大小 [输出] 实际获取到的数据大小，不可为空
            image_w     图像宽度
            image_h     图像高度
  @return  
            ZBCR_SUCCESS    获取成功
            其他错误值      获取失败
*/
ZBCR_EXP ZBCR_STATUS  ZBCR_CALL ZBCR_GetLastImage(unsigned char* image_data, int* imagesize, int* image_w, int* image_h);



/*
    获取API库版本
    @param
        ver_str  输出版本号字符串
    @return  
            ZBCR_SUCCESS
            ZBCR_PARAM_POINT_NULL
*/
ZBCR_EXP ZBCR_STATUS ZBCR_CALL ZBCR_GetApiVersion(char* ver_str);
#ifdef __cplusplus
}
#endif

#endif
